module  IIR_Filter_p8 #(
 parameter  coef_width          = 32 
,parameter  data_in_width       = 16 
,parameter  data_out_width	= 37
,parameter  temp_var_width	= data_out_width+5
)
// H(z) = a(1 + b*z^-1 + b^2*z^-2 + b^3*z^-3 + b^4*z^-4 + b^5*z^-5 + b^6*z^-6 + b^7*z^-7) / (1 - b^8*z^-8)
(
 input	 rstn
,input	 clk
,input	 en
,input   signed	[data_in_width-1 :0]   dinp0	//x(8n+16)
,input   signed	[data_in_width-1 :0]   dinp1 //x(8n+15)
,input   signed	[data_in_width-1 :0]   dinp2 //x(8n+14)
,input   signed	[data_in_width-1 :0]   dinp3 //x(8n+13)
,input   signed	[data_in_width-1 :0]   dinp4 //x(8n+12)
,input   signed	[data_in_width-1 :0]   dinp5 //x(8n+11)
,input   signed	[data_in_width-1 :0]   dinp6 //x(8n+10)
,input   signed	[data_in_width-1 :0]   dinp7 //x(8n+9)

,input   signed	[coef_width-1    :0]   a_re
,input   signed	[coef_width-1    :0]   ab_re
,input   signed	[coef_width-1    :0]   abb_re
,input   signed	[coef_width-1    :0]   ab_pow3_re
,input   signed	[coef_width-1    :0]   ab_pow4_re
,input   signed	[coef_width-1    :0]   ab_pow5_re
,input   signed	[coef_width-1    :0]   ab_pow6_re
,input   signed	[coef_width-1    :0]   ab_pow7_re

,input   signed	[coef_width-1    :0]   b_pow8_re
,output  signed	[data_out_width-1:0]   dout_re		// Re(y(8n-8))
);

wire   signed	[data_in_width-1   :0]   dinp  [7:0];
assign dinp[7] = dinp7;
assign dinp[6] = dinp6;
assign dinp[5] = dinp5;
assign dinp[4] = dinp4;
assign dinp[3] = dinp3;
assign dinp[2] = dinp2;
assign dinp[1] = dinp1;
assign dinp[0] = dinp0;

wire   signed	[coef_width-1   :0]   ab_pow_re  [7:0];
assign ab_pow_re[7] = ab_pow7_re;
assign ab_pow_re[6] = ab_pow6_re;
assign ab_pow_re[5] = ab_pow5_re;
assign ab_pow_re[4] = ab_pow4_re;
assign ab_pow_re[3] = ab_pow3_re;
assign ab_pow_re[2] = abb_re;
assign ab_pow_re[1] = ab_re;
assign ab_pow_re[0] = a_re;



wire	signed	[temp_var_width-1  :0]   x_re [0:7];

wire	signed	[temp_var_width+3  :0]   v_re;
reg	signed	[temp_var_width+3  :0]   v1_re;

wire    signed	[temp_var_width+3  :0]   y_re;
wire    signed	[temp_var_width+3  :0]   y1_re;

wire 	signed	[data_out_width-1:0]   y_re_trunc;

// x[0] = (dinp0 * a_re) delay M 	    = a*x(8n+8)
// x[1] = (dinp1 * ab_re) delay M  	    = a*b*x(8n+7)
// x[2] = (dinp2 * abb_re) delay M  	= a*b^2*x(8n+6)
// x[3] = (dinp3 * ab_pow3_re) delay M  = a*b^3*x(8n+5)
// x[4] = (dinp4 * ab_pow4_re) delay M  = a*b^4*x(8n+4)
// x[5] = (dinp5 * ab_pow5_re) delay M  = a*b^5*x(8n+3)
// x[6] = (dinp6 * ab_pow6_re) delay M  = a*b^6*x(8n+2)
// x[7] = (dinp7 * ab_pow7_re) delay M  = a*b^7*x(8n+1)
genvar i;
generate
    for (i = 0; i < 8; i = i + 1) begin: mult_c_inst
        mult_real #(
            .A_width	(data_in_width	),
            .C_width	(coef_width	),
            .o_width	(temp_var_width	)
        ) inst_c (
            .clk    	(clk		 ),
            .rstn   	(rstn		 ),
            .en     	(en		     ),
            .din      	(dinp[i]	 ),         
            .coef      	(ab_pow_re[i]),
            .dout     	(x_re[i]	 )        
        );
    end
endgenerate


// v1 = sum_{i=0,1,...,7}{x[i]} delay M = sum_{i=0,1,...,7}{a*b^i*x(8n-i)}
assign	v_re       =   x_re[0] + x_re[1] +x_re[2] +x_re[3] +x_re[4] +x_re[5] +x_re[6] +x_re[7];

always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
                v1_re    <=    'h0;
 		 end 
	 else if(en)
		 begin
                v1_re    <=    v_re;
		  end
	 else
		 begin
                v1_re    <=    v1_re;
		  end


// y1 = (b^8 * y) delay M = b^8*y(8n-8)
// y = v1 + y1 = sum_{i=0,1,...,7}{a*b^i*x(8n-i)} + b^8*y(8n-8) = y(8n)
mult_real
#(
	 .A_width	(temp_var_width+4	) 
	,.C_width	(coef_width		    ) 
	,.o_width	(temp_var_width+4 	) 
)
inst_c9 (
         .clk		(clk    	),
         .rstn		(rstn   	),
         .en		(en     	),
         .din		(y_re		),
         .coef		(b_pow8_re	),
         .dout		(y1_re     	)
);

assign	y_re       =   v1_re + y1_re;

// dout = round(y) delay M = round(y(8n-8))
trunc #(
	 .diw	(temp_var_width+4		)
	,.msb	(temp_var_width-1		)
	,.lsb	(temp_var_width-data_out_width	)
) round_u1 (clk, rstn, en, y_re, y_re_trunc);


assign	dout_re = y_re_trunc;

endmodule

